import os

disabled = os.getenv("DISABLE_UJSON")
if not disabled:
    try:
        import ujson as json
    except ImportError:
        import json  # type: ignore
else:
    import json  # type: ignore


def loads(data) -> dict:
    return json.loads(data)


def dumps(data) -> str:
    return json.dumps(data, ensure_ascii=False)



import asyncio
import io
import ssl
from pathlib import Path
from typing import Optional, Tuple, Union

import certifi
from aiohttp import ClientSession, FormData, TCPConnector
from aiohttp.typedefs import StrOrURL

from .utils import json


class BaseClient:
    def __init__(self):
        self._session: Optional[ClientSession] = None

    def _get_session(self):
        if isinstance(self._session, ClientSession) and not self._session.closed:
            return self._session

        ssl_context = ssl.create_default_context(cafile=certifi.where())
        connector = TCPConnector(ssl=ssl_context)

        self._session = ClientSession(
            connector=connector,
            json_serialize=json.dumps,
        )
        return self._session

    async def _make_request(
        self, method: str, url: StrOrURL, **kwargs
    ) -> Tuple[int, dict]:
        session = self._get_session()
        result = await session.request(method, url, **kwargs)
        return result.status, await result.json(loads=json.loads)

    def _prepare_form(self, file: Union[str, Path, io.IOBase]):
        form = FormData()
        form.add_field("file", self._prepare_file(file))
        return form

    @staticmethod
    def _prepare_file(file: Union[str, Path, io.IOBase]):
        if isinstance(file, str):
            return open(file, "rb")

        if isinstance(file, io.IOBase):
            return file

        if isinstance(file, Path):
            return file.open("rb")

        raise TypeError("Not supported file type.")

    async def close(self):
        if not isinstance(self._session, ClientSession):
            return

        if self._session.closed:
            return

        await self._session.close()
        await asyncio.sleep(0.25)



class TrueCheckerException(Exception):
    pass


class BadRequest(TrueCheckerException):
    pass


class Unauthorized(TrueCheckerException):
    pass


class AlreadyRunning(TrueCheckerException):
    pass


class ValidationError(TrueCheckerException):
    pass



import io
from pathlib import Path
from typing import Optional, Union

from .base import BaseClient
from .const import HTTPMethods
from .exceptions import (
    AlreadyRunning,
    BadRequest,
    Unauthorized,
)
from .models import CheckJob, Profile

API_HOST = "https://checker.trueweb.app/api"


class TrueChecker(BaseClient):
    API_VERSION = "1.1.0"

    def __init__(self, token: str, api_host: Optional[str] = None):
        super().__init__()
        self.__token = token
        self._api_host = api_host or API_HOST

    async def check_profile(
        self,
        file: Union[str, Path, io.IOBase],
        delay: Optional[float] = None,
    ) -> CheckJob:
        """ Bot check request. """
        method = HTTPMethods.PUT
        url = f"{self._api_host}/profile/{self.__token}"

        # prepare params
        params = {}
        if delay is not None:
            params["delay"] = delay
        form = self._prepare_form(file)

        status, data = await self._make_request(method, url, params=params, data=form)
        if status != 200:
            if status == 401:
                raise Unauthorized(data["message"])
            if status == 409:
                raise AlreadyRunning(data["message"])
            raise BadRequest(data["message"])
        return CheckJob(**data)

    async def get_profile(self, username: str) -> Profile:
        """ Returns checked bot profile on success. """
        method = HTTPMethods.GET
        url = f"{self._api_host}/profile/{username}"
        status, profile_data = await self._make_request(method, url)
        return Profile(**profile_data)

    async def get_job_status(self, job_id: str) -> CheckJob:
        """ Returns current job status. """
        method = HTTPMethods.GET
        url = f"{self._api_host}/job/{job_id}"
        status, job_data = await self._make_request(method, url)
        return CheckJob(**job_data)

    async def cancel_job(self, job_id: str) -> CheckJob:
        """ Cancel running Job. """
        method = HTTPMethods.DELETE
        url = f"{self._api_host}/job/{job_id}"
        status, job_data = await self._make_request(method, url)
        return CheckJob(**job_data)
